﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NativeWifi;

namespace AirFusion.Dandelion
{
	/// <summary>
	/// Represents the wireless access point.
	/// </summary>
	class WirelessUtils
	{
		/// <summary>
		/// Gets the SSID list of nearby wireless access points.
		/// </summary>
		/// <param name="ssidList"> SSID list of nearby wireless access points. </param>
		/// <exception cref="Exception"></exception>
		public void GetNearbyWirelessAccessPoints(out List<string> ssidList)
		{
			ssidList = new List<string>();
			try
			{
				//WlanClient wlanClient = new WlanClient();
				WlanClient wlanClient = WlanClient.Client;
				foreach (WlanClient.WlanInterface wlanInterface in wlanClient.Interfaces)
				{
					Wlan.WlanAvailableNetwork[] networks = wlanInterface.GetAvailableNetworkList(0);

					foreach (Wlan.WlanAvailableNetwork network in networks)
					{
						Wlan.Dot11Ssid ssid = network.dot11Ssid;

						string networkName = Encoding.ASCII.GetString(ssid.SSID, 0, (int)ssid.SSIDLength);
						ssidList.Add(networkName);
						//Console.WriteLine(networkName + ": " + network.wlanSignalQuality + "%");
					}
				}
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// Gets the BSSID list of nearby wireless access points.
		/// </summary>
		/// <param name="bssidList"> BSSID list of nearby wireless access points. </param>
		/// <exception cref="Exception"></exception>
		public void GetNearbyWirelessAccessPoints(out List<byte[]> bssidList)
		{
			bssidList = new List<byte[]>();
			try
			{
				//WlanClient wlanClient = new WlanClient();
				WlanClient wlanClient = WlanClient.Client;
				foreach (WlanClient.WlanInterface wlanInterface in wlanClient.Interfaces)
				{
					Wlan.WlanBssEntry[] wlanBssEntries = wlanInterface.GetNetworkBssList();
					foreach (Wlan.WlanBssEntry wlanBssEntry in wlanBssEntries)
					{
						byte[] macAddr = wlanBssEntry.dot11Bssid;
						bssidList.Add(macAddr);
					}
				}
			}
			catch
			{
				throw;
			}
		}

	}
}
