﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion
{
	#region Define event triggered from App

	/// <summary>
	/// Enumeration for event type from application
	/// </summary>
	public enum AppEventType
	{
		Pairing,
		PairingFinished,
		PairedDeviceListShow,
		InitialStatus,
		UploadFilePercentage,
		DownloadFilePercentage,
		Disconnected,
		NoBTCapability,
		RotateDirection,
		PairedDeviceUpdate,
		LaunchScreenCastWindow,
		ConnectScreenCasting,
		RestartScreenCasting,
		Reconnecting,

		//CloseWindow,
		WaitForPairingTimeout,

		PairAgain,
		Unpair,
		ScreenOff,
		StartReceiving,
		ScreenCastTCPDIsconnected,

		#region 0422 Johnson add for VNC Upload File use

		UploadAllFileStart,
		UploadAllFileEnd,

		#endregion 0422 Johnson add for VNC Upload File use

		//0422 Johnson add for VNC transfer file error handle
		VNCTransferFileError,

		//0508 Johnson add for s3/s4/screenLock event do special handle
		CloseWindowBySystemSleep,

		//0616 Progress Step
		NextProgress,

		//0630 Johnson Add RotateButton Status
		RotateButtonStatus,

		//0703 Johnson add For Non-Acer Fail case
		NonAcerFlowFailStatus,
	}

	/// <summary>
	/// Class for event argument from application
	/// </summary>
	public class AppEventArgs : EventArgs
	{
		public AppEventArgs(AppEventType eventType, object data = null, object args = null)
		{
			this._eventType = eventType;
			this._data = data;
			this.args = args;
		}

		private AppEventType _eventType;

		public AppEventType EventType
		{
			get
			{
				return _eventType;
			}
		}

		private object _data = null;

		public object Data
		{
			get
			{
				return _data;
			}
		}

		private object args = null;

		public object Args
		{
			get
			{
				return args;
			}
		}
	}

	#endregion Define event triggered from App

	#region Define event triggered from Window

	/// <summary>
	/// Enumeration for event type from window
	/// </summary>
	public enum WindowEventType
	{
		PairingResponse,
		UnpairResponse,
		VNCSetScreenSize,
		UploadFiles,
		WakeUp,
		LaunchScreenCasting,
		CloseScreenCasting,
		RestartScreenCasting,
		ShowDeviceManager,
		ExistedLaunch,
		SetDiscoverable,
		NavigationBackClicked,
		NavigationHomeClicked,
		NavigationRecentClicked,
		SwitchRotationClicked,
		SwitchKeyboardClicked,
		GetSnapshot,
		SetAccountPassword,
		DeleteAccountPassword,
		CheckReceiveData,
		PairAgain,
		Exit,
		TCPDisconnected,
	}

	/// <summary>
	/// Class for event argument from window
	/// </summary>
	public class WindowEventArgs : EventArgs
	{
		public WindowEventArgs(WindowEventType eventType, object data = null, object args = null)
		{
			this._eventType = eventType;
			this._data = data;
			this.args = args;
		}

		private WindowEventType _eventType;

		public WindowEventType EventType
		{
			get
			{
				return _eventType;
			}
		}

		private object _data = null;

		public object Data
		{
			get
			{
				return _data;
			}
		}

		private object args = null;

		public object Args
		{
			get
			{
				return args;
			}
		}
	}

	#endregion Define event triggered from Window

	#region Define event triggered from User Control

	/// <summary>
	/// Enumeration for event type from user control
	/// </summary>
	public enum UserControlEventType
	{
		None,
		PairingUnsccessAction,
		UnpairRequest,
		UnpairRequestResult,
		PageBack,
		SetPWD,
		SetPWDResult,
		ChangePWD,
		ChangePWDResult,
		DeletePWD,
		WaitForPairingTimeout,
		ShowQRCode,
		SetDiscoverableValue,
		TapToLaunchDevice,
		ReconnectionScreenCast,
		StopScreenCast,
		StopScreenHint,
		ShowPairingPage,

		DetailDebugMessage,
	}

	/// <summary>
	/// Class for event argument from user control
	/// </summary>
	public class UserControlEventArgs : EventArgs
	{
		public UserControlEventArgs(UserControlEventType eventType, object data = null, object args = null)
		{
			this._eventType = eventType;
			this._data = data;
			this._args = args;
		}

		private UserControlEventType _eventType;

		public UserControlEventType EventType
		{
			get
			{
				return _eventType;
			}
		}

		private object _data = null;

		public object Data
		{
			get
			{
				return _data;
			}
		}

		private object _args = null;

		public object Args
		{
			get
			{
				return _args;
			}
		}
	}

	/// <summary>
	/// Enumeration for page type in UI
	/// </summary>
	public enum PageType
	{
		DeviceSettings,
		QRCode,
		GlobalSettings,
	}

	#endregion Define event triggered from User Control

	#region Define event triggered from RTSPController

	/// <summary>
	/// Enumeration for event type from RTSP controller
	/// </summary>
	public enum RTSPControllerEventType
	{
		RTSPSocketDisconnected,
	}

	/// <summary>
	/// Class for event argument from RTSP controller
	/// </summary>
	public class RTSPControllerEventArgs : EventArgs
	{
		public RTSPControllerEventArgs(RTSPControllerEventType eventType)
		{
			_eventType = eventType;
		}

		private RTSPControllerEventType _eventType;

		public RTSPControllerEventType EventType
		{
			get
			{
				return _eventType;
			}
		}
	}

	#endregion Define event triggered from RTSPController
}