﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AirFusion
{
	/// <summary>
	/// Interaction logic for PairedDeviceTab.xaml
	/// </summary>
	public partial class PairedDeviceTab : UserControl
	{
		#region Events

		public event EventHandler<UserControlEventArgs> UserControlEvent;

		#endregion Events

		public ICommand ClickHyperlinkPairingPage { get; private set; }

		#region Constructor & Dispose

		public PairedDeviceTab()
		{
			// These commands needs to be declared and initialized before initializing UI components because there are UI components uses them with binding.
			this.ClickHyperlinkPairingPage = new DelegateCommand(this.ShowPairingPage, null);

			InitializeComponent();

			this.DataContext = this;
			this.SetNoDeviceDetails();
		}

		#endregion Constructor & Dispose

		#region Callbacks

		private void UserControlEventCallback(object sender, UserControlEventArgs e)
		{
			try
			{
				if (e.EventType == UserControlEventType.PageBack)
				{
					if (e.Data is PageType)
					{
						PageType type = (PageType)e.Data;

						switch (type)
						{
							case PageType.DeviceSettings:
								{
									this.Dispatcher.Invoke(delegate()
									{
										this.grdDeviceList.Visibility = System.Windows.Visibility.Visible;
										//this.lclNoDevicePage.Visibility = System.Windows.Visibility.Collapsed;
										this.lclDeviceSettingsPage.TargetDeviceInfo = null;
										this.lclDeviceSettingsPage.CurrentUserAccount = string.Empty;
										this.lclDeviceSettingsPage.Visibility = System.Windows.Visibility.Collapsed;
									});
								}
								break;

							case PageType.GlobalSettings:
								{
									this.Dispatcher.Invoke(delegate()
									{
										this.grdDeviceList.Visibility = System.Windows.Visibility.Visible;
										this.lclGlobalSettingsPage.TargetSettings = null;
										this.lclGlobalSettingsPage.Visibility = System.Windows.Visibility.Collapsed;
									});
								}
								break;
						}
					}
				}
				else if (e.EventType == UserControlEventType.SetPWD || e.EventType == UserControlEventType.ChangePWD)
				{
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, e);
					}
				}
				else if (e.EventType == UserControlEventType.DeletePWD)
				{
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.DeletePWD, e.Data));
					}
				}
				else if (e.EventType == UserControlEventType.UnpairRequest)
				{
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.UnpairRequest, e.Data));
					}
				}
				else if (e.EventType == UserControlEventType.ShowPairingPage)
				{
					if (null != this.UserControlEvent)
					{
						this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.ShowPairingPage, e.Data));
					}
				}
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		#endregion Callbacks

		#region UI Event Callbacks

		private void ShowPairingPage(object parameter)
		{
			LogtoFile.WriteLog("", Configs.DebugLevelValue, "Call ShowPairingPage()", "");

			if (null != this.UserControlEvent)
			{
				this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.ShowPairingPage));
			}
		}

		private void btnDeviceSettings_Click(object sender, RoutedEventArgs e)
		{
			AppDeviceInfo deviceInfo = (sender as Button).DataContext as AppDeviceInfo;

			this.grdDeviceList.Visibility = System.Windows.Visibility.Collapsed;

			this.lclDeviceSettingsPage.TargetDeviceInfo = deviceInfo;
			this.lclDeviceSettingsPage.CurrentUserAccount = Utils.GetAccountDisplayName();
			this.lclDeviceSettingsPage.Visibility = System.Windows.Visibility.Visible;
		}

		private void btnGlobalSettings_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				this.grdDeviceList.Visibility = System.Windows.Visibility.Collapsed;

				this.lclGlobalSettingsPage.TargetSettings = Configs.ApplicationReference.Settings;
				this.lclGlobalSettingsPage.Visibility = System.Windows.Visibility.Visible;
			}
			catch (Exception ex)
			{
				LogtoFile.WriteLog("", Configs.DebugLevelValue, ex.ToString(), "");
			}
		}

		private void TextBlock_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			AppDeviceInfo deviceInfo = (sender as TextBlock).DataContext as AppDeviceInfo;

			this.UserControlEvent(this, new UserControlEventArgs(UserControlEventType.TapToLaunchDevice, deviceInfo));
		}

		#endregion UI Event Callbacks

		public void SetInitialPairedDeviceTabInfo()
		{
			try
			{
				var targetItem = this.lstvPairedDevices.Items.Cast<AppDeviceInfo>().Where(i => i.Device.InstanceID == Configs.ApplicationReference.ConnectingDeviceInfo.Device.InstanceID).FirstOrDefault();
				if (null != targetItem)
				{
					this.lstvPairedDevices.SelectedItem = targetItem;
				}
			}
			catch (Exception ex)
			{ }
		}

		private void UserControl_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			if (this.Visibility == Visibility.Visible)
				this.SetInitialPairedDeviceTabInfo();
		}

		private void SetNoDeviceDetails()
		{
			try
			{
				Run run = new Run(this.TryFindResource("txtPairingHyperlink") as string);
				Hyperlink hyperlink = new Hyperlink(run);
				hyperlink.Command = this.ClickHyperlinkPairingPage;
				hyperlink.Foreground = this.TryFindResource("StepHyperlinkFontColor") as SolidColorBrush;

				string tmp = this.TryFindResource("txtNoDeviceDetail") as string;

				string[] tmp2 = tmp.Split(new string[] { "{0}" }, StringSplitOptions.None);

				this.txtbNoDeviceDetails.Inlines.Add(tmp2[0]);
				this.txtbNoDeviceDetails.Inlines.Add(hyperlink);
				this.txtbNoDeviceDetails.Inlines.Add(tmp2[1]);
			}
			catch (Exception ex)
			{ }
		}
	}
}