﻿namespace AirFusion.Common.Wmi.Cim
{
	/// <summary>
	/// The CIM_Card class represents a type of physical container that can be plugged into another card or hosting board, or is itself a hosting board/motherboard in a chassis.
	/// This class includes any package that is capable of carrying signals and providing a mounting point for physical components, such as chips or other physical packages, such as other cards.
	/// </summary>
	[ParameterValue("CIM_Card")]
	public class WmiCard : WmiPhysicalPackage
	{
		/// <summary>
		/// Gets a value that indicates if this card is a motherboard or, more generically, a baseboard in a chassis.
		/// </summary>
		[ParameterValue("HostingBoard")]
		public bool? HostingBoard { get; internal set; }

		/// <summary>
		/// Gets the free-form string that describes the ways in which the card is physically unique from other cards.
		/// This property only has meaning when the corresponding Boolean property, SpecialRequirements, is set to TRUE.
		/// </summary>
		[ParameterValue("RequirementsDescription")]
		public string RequirementsDescription { get; internal set; }

		/// <summary>
		/// Gets a value that indicates at least one daughterboard or auxiliary card is required to function properly.
		/// </summary>
		[ParameterValue("RequiresDaughterBoard")]
		public bool? RequiresDaughterBoard { get; internal set; }

		/// <summary>
		/// Gets the free-form string that describes the slot positioning, typical usage, restrictions, individual slot spacings, or other pertinent information for the slots on a card.
		/// </summary>
		[ParameterValue("SlotLayout")]
		public string SlotLayout { get; internal set; }

		/// <summary>
		/// Gets a value that indicates if the card is physically unique from other cards of the same type and, therefore, requires a special slot.
		/// For example, a double-wide card requires two slots.
		/// Another example is when a certain card can be used for the same general function as other cards, but requires a special slot (extra long, for example);
		/// whereas, other cards can be placed in any available slot. If TRUE, the corresponding RequirementsDescription property should specify the nature of the uniqueness or purpose of the card.
		/// </summary>
		[ParameterValue("SpecialRequirements")]
		public bool? SpecialRequirements { get; internal set; }

		/// <summary>
		/// Initializes the object.
		/// </summary>
		public WmiCard()
		{
			this.HostingBoard = null;
			this.RequirementsDescription = null;
			this.RequiresDaughterBoard = null;
			this.SlotLayout = null;
			this.SpecialRequirements = null;
		}
	}
}