﻿using System;
using System.ComponentModel;

namespace AirFusionCustomToast
{
	#region Enumerators

	public enum ToastType
	{
		None,
		GeneralToast,
		PhoneCallToast,
	}

	public enum PhoneCallToastActionType
	{
		Answer,
		Decline,
		Mute,
		End,
		Message,
		None
	};

	public enum PhoneCallState
	{
		None,
		Incoming,
		OnTheLine,
	};

	#endregion

	public class ToastUIElement : INotifyPropertyChanged
	{
		#region Private Members

		private bool _launchable = false;
		private bool _removable = false;
		private string _title = string.Empty;
		private string _contentLine1 = string.Empty;
		private string _contentLine2 = string.Empty;
		private Guid _notificationKey = Guid.Empty;
		private string _deviceInstanceID = string.Empty;
		private System.Windows.Media.Imaging.BitmapSource _bitmapIcon = null;

		#endregion

		#region Public Properties

		public bool Launchable
		{
			get
			{
				return _launchable;
			}

			set
			{
				_launchable = value;
				this.OnPropertyChanged("Launchable");
			}
		}

		public bool Removable
		{
			get
			{
				return _removable;
			}

			set
			{
				_removable = value;
				this.OnPropertyChanged("Removable");
			}
		}

		public string Title
		{
			get
			{
				return _title;
			}

			set
			{
				_title = value;
				this.OnPropertyChanged("Title");
			}
		}

		public string ContentLine1
		{
			get
			{
				return _contentLine1;
			}

			set
			{
				_contentLine1 = value;
				this.OnPropertyChanged("ContentLine1");
			}
		}

		public string ContentLine2
		{
			get
			{
				return _contentLine2;
			}

			set
			{
				_contentLine2 = value;
				this.OnPropertyChanged("ContentLine2");
			}
		}

		public Guid NotificationKey
		{
			get
			{
				return _notificationKey;
			}

			set
			{
				_notificationKey = value;
				this.OnPropertyChanged("NotificationKey");
			}
		}

		public string DeviceInstanceID
		{
			get
			{
				return _deviceInstanceID;
			}

			set
			{
				_deviceInstanceID = value;
				this.OnPropertyChanged("DeviceInstanceID");
			}
		}

		public System.Windows.Media.Imaging.BitmapSource BitmapIcon
		{
			get
			{
				return _bitmapIcon;
			}

			set
			{
				_bitmapIcon = value;
				this.OnPropertyChanged("BitmapIcon");
			}
		}

		#endregion

		#region Events

		public event PropertyChangedEventHandler PropertyChanged;

		#endregion

		#region Methods
		#region Protected Methods

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion
		#endregion
	}
}
