﻿using System;

namespace AirFusionCustomToast
{
	#region Define event triggered from Window

    /// <summary>
    /// Represents the trigger type of the controls in the toast
    /// </summary>
	public enum ToastEventType
	{
        /// <summary>
        /// The toast dialog is triggered
        /// </summary>
		ToastActivated,

        /// <summary>
        /// The toast launch button is triggered
        /// </summary>
        ToastLaunched,

        /// <summary>
        /// The toast ignore button is triggered
        /// </summary>
        ToastIgnored,

        /// <summary>
        /// The toast quick action button is triggered
        /// </summary>
        ToastActionLaunched,
	}

    /// <summary>
    /// Represents the intent to do when the control in the toast is triggered
    /// </summary>
    public enum ToastIntent
    {
        /// <summary>
        /// The screen cast should be shown
        /// </summary>
        ShowScreenCasting,
    }

    /// <summary>
    /// Represents the event arguments when the toast is triggered
    /// </summary>
	public class ToastEventArgs : EventArgs
	{
        /// <summary>
        /// Initializes a new instance of the <see cref="ToastEventArgs"/> class.
        /// </summary>
        public ToastEventArgs(ToastEventType eventType, ToastIntent intent, object data = null, object args = null)
		{
            this.EventType = eventType;
			this.Data = data;
            this.Args = args;
            this.Intent = intent;
		}

        #region Propoerties

        /// <summary>
        /// Gets the triggered event type
        /// </summary>
        public ToastEventType EventType { get; private set; }

        /// <summary>
        /// Gets the intent of the event
        /// </summary>
        public ToastIntent Intent { get; private set; }

        /// <summary>
        /// Gets the data of the event
        /// </summary>
        public object Data { get; private set; }

        /// <summary>
        /// Gets the additional args of the event
        /// </summary>
        public object Args { get; private set; }

        #endregion
    }

	#endregion
}
