﻿using Newtonsoft.Json;
using System;
using System.Net;

namespace AirFusion.Common.JsonConverters
{
	internal class IPAddressConverter : JsonConverter
	{
		public override bool CanConvert(Type objectType)
		{
			return typeof(IPAddress) == objectType;
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
		{
			if (CanConvert(objectType) == false)
			{
				throw new ArgumentException("Not supported object type: " + objectType.GetFullName());
			}

			IPAddress ret = null;
			string stringValue = reader.Value != null ? reader.Value.ToString() : null;

			switch (reader.TokenType)
			{
				case JsonToken.Null:
					return ret;

				case JsonToken.String:
					if (!string.IsNullOrEmpty(stringValue))
					{
						IPAddress.TryParse(stringValue, out ret);
					}
					break;

				default:
					throw new ArgumentException("Invalid token type.");
			}

			if (ret == null)
			{
				throw new ArgumentException("Invalid IP address: " + stringValue);
			}

			return ret;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
		{
			if (value is IPAddress)
			{
				writer.WriteValue(((IPAddress)value).ToString());
			}
			else
			{
				writer.WriteNull();
			}
		}
	}
}
