﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Common.ProximityDetection
{
	/// <summary>
	/// Represents the methods that will handle the callback of StartListen().
	/// </summary>
	/// <param name="args">The arguments.</param>
	public delegate void ProximityDetectionCallback(IProximityDetectionCallbackArgs args);

	/// <summary>
	/// Represents the interface of callback arguments of proximity detection.
	/// </summary>
	public interface IProximityDetectionCallbackArgs
	{
		/// <summary>
		/// Gets the identity.
		/// </summary>
		Guid GetIdentity();

		/// <summary>
		/// Gets the additional data.
		/// </summary>
		byte[] GetAdditionalData();
	}

	/// <summary>
	/// Represents the interface of proximity detection.
	/// </summary>
	public interface IProximityDetection
	{
		/// <summary>
		/// Starts the detection.
		/// </summary>
		/// <param name="callback">The callback.</param>
		void StartDetection(ProximityDetectionCallback callback);

		/// <summary>
		/// Stops the detection.
		/// </summary>
		void StopDetection();
	}
}