﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public sealed partial class UserDeviceManager
	{
		internal sealed partial class DandelionAdapter
		{
			/// <summary>
			/// Represents the callback arguments of a nearby scan finished event.
			/// </summary>
			public class NearbyScanCallbackArg
			{
				/// <summary>
				/// Gets the instance ID of found device.
				/// </summary>
				public Guid? InstanceID { get; private set; }

				/// <summary>
				/// Gets the profile of found device.
				/// </summary>
				public InstanceProfile Profile { get; private set; }

				/// <summary>
				/// Gets a value indicating whether the operation is completed.
				/// </summary>
				public bool IsCompleted { get; private set; }

				/// <summary>
				/// Gets a value indicating whether the operation is faulted.
				/// </summary>
				public bool IsFaulted
				{
					get
					{
						return this.Exception != null;
					}
				}

				/// <summary>
				/// Gets the exception if any.
				/// </summary>
				public Exception Exception { get; private set; }

				/// <summary>
				/// Initializes a new instance of the <see cref="NearbyScanCallbackArg"/> class.
				/// </summary>
				/// <param name="instanceID">The instance ID.</param>
				/// <param name="profile">The profile.</param>
				/// <param name="isCompleted">A value indicating whether the operation is completed.</param>
				internal NearbyScanCallbackArg(Guid instanceID, InstanceProfile profile, bool isCompleted)
				{
					this.InstanceID = instanceID;
					this.Profile = profile;
					this.IsCompleted = isCompleted;
					this.Exception = null;
				}

				/// <summary>
				/// Initializes a new instance of the <see cref="NearbyScanCallbackArg"/> class.
				/// </summary>
				/// <param name="exception">The exception.</param>
				/// <param name="isCompleted">A value indicating whether the operation is completed.</param>
				internal NearbyScanCallbackArg(Exception exception, bool isCompleted)
				{
					this.InstanceID = null;
					this.Profile = null;
					this.IsCompleted = isCompleted;
					this.Exception = exception;
				}

				/// <summary>
				/// Returns a <see cref="System.String" /> that represents this instance.
				/// </summary>
				/// <returns>
				/// A <see cref="System.String" /> that represents this instance.
				/// </returns>
				public override string ToString()
				{
					StringBuilder builder = new StringBuilder();
					builder.AppendLine(this.GetType().GetFullName() + ".InstanceID: " + (this.InstanceID.HasValue ? this.InstanceID.Value.GetStringFormat() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".Profile: " + (this.Profile != null ? "\r\n" + this.Profile.ToString() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".Exception: " + (this.Exception != null ? this.Exception.ToString() : "N/A"));
					builder.AppendLine(this.GetType().GetFullName() + ".IsCompleted: " + this.IsCompleted);
					builder.AppendLine(this.GetType().GetFullName() + ".IsFaulted: " + this.IsFaulted);

					return builder.ToString().TrimEnd();
				}
			}

			/// <summary>
			/// Represents the methods that will handle the callback of a nearby scan.
			/// </summary>
			/// <param name="eventArgs">The <see cref="NearbyScanCallbackArg"/> instance containing the callback argument data.</param>
			public delegate void NearbyScanCallback(NearbyScanCallbackArg eventArgs);
		}
	}
}