﻿using AirFusion.Common;
using System;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the event arguments of pairing responsed event.
	/// </summary>
	public sealed class PairingResponsedEventArgs : EventArgs
	{
		#region Enumerations

		/// <summary>
		/// Specifies the type of pairing respose.
		/// </summary>
		public enum PairingResponseType
		{
			/// <summary>
			/// Reserved.
			/// </summary>
			Reserved,

			/// <summary>
			/// Signifies an unknown response type.
			/// </summary>
			Unknown = -1
		}

		#endregion Enumerations

		#region Properties

		/// <summary>
		/// Gets the device.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		/// <summary>
		/// Gets the type of the response.
		/// </summary>
		public PairingResponseType ResponseType { get; private set; }

		/// <summary>
		/// Gets a value indicates whether the pairing request is accepted.
		/// </summary>
		public bool? Accepted { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingResponsedEventArgs" /> class.
		/// </summary>
		internal PairingResponsedEventArgs()
		{
			this.Device = null;
			this.ResponseType = PairingResponseType.Unknown;
			this.Accepted = null;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="PairingResponsedEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device.</param>
		/// <param name="responseType">The type of the pairing response.</param>
		/// <param name="accepted">A value indicates whether the pairing request is accepted.</param>
		internal PairingResponsedEventArgs(AFUserDevice device, PairingResponseType responseType, bool? accepted)
		{
			this.Device = device;
			this.ResponseType = responseType;
			this.Accepted = accepted;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));
			builder.AppendLine(this.GetType().GetFullName() + ".ResponseType: " + this.ResponseType);
			builder.AppendLine(this.GetType().GetFullName() + ".Accepted: " + (this.Accepted != null ? this.Accepted.ToString() : "N/A"));

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}