﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace AirFusion.Manager.DeviceManager
{
	/// <summary>
	/// Represents the result of ReconfirmAddressAsync().
	/// </summary>
	public sealed class ReconfirmAddressResult
	{
		#region Private/Protected/Internal

		internal Dictionary<IPEndPoint, ConnectionStatus> EndPointConnectionStatus { get; set; }

		#endregion Private/Protected/Internal

		/// <summary>
		/// Gets the connection status dictionary of each address.
		/// </summary>
		public IDictionary<IPAddress, ConnectionStatus> ConnectionStatus
		{
			get
			{
				try
				{
					Debug.Assert(this.EndPointConnectionStatus != null);
					return this.EndPointConnectionStatus.ToDictionary(key => key.Key.Address, element => element.Value);
				}
				catch
				{
					return new Dictionary<IPAddress, ConnectionStatus>();
				}
			}
		}

		/// <summary>
		/// Gets the collection of addresses.
		/// </summary>
		public ICollection<IPAddress> Addresses
		{
			get
			{
				try
				{
					return this.ConnectionStatus.Keys;
				}
				catch
				{
					return new List<IPAddress>();
				}
			}
		}

		/// <summary>
		/// Gets the first connectable address.
		/// </summary>
		public IPAddress FirstConnectableAddress
		{
			get
			{
				try
				{
					Debug.Assert(this.EndPointConnectionStatus != null);
					return this.EndPointConnectionStatus.Where(item => item.Value == DeviceManager.ConnectionStatus.Connectable).Select(item => item.Key.Address).FirstOrDefault();
				}
				catch
				{
					return null;
				}
			}
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="ReconfirmAddressResult"/> class.
		/// </summary>
		internal ReconfirmAddressResult()
		{
			this.EndPointConnectionStatus = new Dictionary<IPEndPoint, ConnectionStatus>();
		}
	}
}