﻿using AirFusion.Common;
using AirFusion.Common.RegistryHolders;
using System.Text;

namespace AirFusion.Manager.DeviceManager
{
	public sealed partial class AFUserDevice
	{
		public sealed partial class ServicesInfo
		{
			public sealed partial class ScreencastInfo
			{
				/// <summary>
				/// Represents the two dimensions of resolution.
				/// </summary>
				public struct ScreenResolution
				{
					#region Private

					private int _width;
					private int _height;

					#endregion Private

					/// <summary>
					/// Gets the width.
					/// </summary>
					[RegistryStoreTypeAttribute(StoreType.Value)]
					public int Width
					{
						get { return this._width; }
						private set { this._width = value; }
					}

					/// <summary>
					/// Gets the height.
					/// </summary>
					[RegistryStoreTypeAttribute(StoreType.Value)]
					public int Height
					{
						get { return this._height; }
						private set { this._height = value; }
					}

					/// <summary>
					/// Initializes a new instance of the <see cref="ScreenResolution"/> struct.
					/// </summary>
					/// <param name="width">The width.</param>
					/// <param name="height">The height.</param>
					public ScreenResolution(int width, int height)
					{
						this._width = width;
						this._height = height;
					}

					/// <summary>
					/// Sets the width.
					/// </summary>
					/// <param name="width">The width.</param>
					public void SetWidth(int width)
					{
						this.Width = width;
					}

					/// <summary>
					/// Sets the height.
					/// </summary>
					/// <param name="height">The height.</param>
					public void SetHeight(int height)
					{
						this.Height = height;
					}

					#region Override

					/// <summary>
					/// Determines whether the specified object is equal to the current object.
					/// </summary>
					/// <param name="obj">The object to compare with the current object.</param>
					/// <returns></returns>
					public override bool Equals(object obj)
					{
						return (obj is ScreenResolution && this == (ScreenResolution)obj);
					}

					/// <summary>
					/// Returns the hash code of the instance.
					/// </summary>
					/// <returns></returns>
					public override int GetHashCode()
					{
						return base.GetHashCode() ^ this.Width.GetHashCode() ^ this.Height.GetHashCode();
					}

					/// <summary>
					/// Implements the operator ==.
					/// </summary>
					/// <param name="x">The first object.</param>
					/// <param name="y">The second object.</param>
					public static bool operator ==(ScreenResolution x, ScreenResolution y)
					{
						return x.Width == y.Width && x.Height == y.Height;
					}

					/// <summary>
					/// Implements the operator !=.
					/// </summary>
					/// <param name="x">The first object.</param>
					/// <param name="y">The second object.</param>
					public static bool operator !=(ScreenResolution x, ScreenResolution y)
					{
						return !(x == y);
					}

					/// <summary>
					/// Returns a <see cref="System.String" /> that represents this instance.
					/// </summary>
					/// <returns>
					/// A <see cref="System.String" /> that represents this instance.
					/// </returns>
					public override string ToString()
					{
						return this.Width + "*" + this.Height;
					}

					#endregion Override
				}
			}
		}
	}
}