﻿using AirFusion.Common;
using Newtonsoft.Json;
using System.Text;

namespace AirFusion.WebService.Adapters
{
	internal partial class DeviceInformation
	{
		/// <summary>
		/// Represents the display information of the device.
		/// </summary>
		public partial class DisplayInfo
		{
			/// <summary>
			/// Gets or sets the rotation of the display.
			/// </summary>
			[JsonProperty("Rotation"), JsonConverter(typeof(AirFusion.Common.JsonConverters.EnumConverter))]
			public DisplayRotation? Rotation { get; internal set; }

			/// <summary>
			/// Gets or sets the width of the display.
			/// </summary>
			[JsonProperty("Width")]
			public int? Width { get; internal set; }

			/// <summary>
			/// Gets or sets the width of the display.
			/// </summary>
			[JsonProperty("Height")]
			public int? Height { get; internal set; }

			/// <summary>
			/// Gets or sets a value that indicates if the navigation bar is displayed virtually.
			/// </summary>
			[JsonProperty("Navigation")]
			public bool? Navigation { get; internal set; }

			/// <summary>
			/// Gets or sets a value that indicates if the screen is ON or not.
			/// </summary>
			[JsonProperty("ScreenOn")]
			public bool? ScreenOn { get; internal set; }

			/// <summary>
			/// Gets or sets a value that indicates if the AutoRotation is ON or not.
			/// </summary>
			[JsonProperty("AutoRotation")]
			public bool? AutoRotation { get; internal set; }

			#region Private/Protected/Internal

			/// <summary>
			/// Initializes a new instance of the <see cref="DisplayInfo"/> class.
			/// </summary>
			internal DisplayInfo()
			{
				this.Rotation = null;
				this.Width = null;
				this.Height = null;
				this.Navigation = null;
				this.ScreenOn = null;
				this.AutoRotation = null;
			}

			#endregion Private/Protected/Internal

			/// <summary>
			/// Returns a <see cref="System.String" /> that represents this instance.
			/// </summary>
			/// <returns>
			/// A <see cref="System.String" /> that represents this instance.
			/// </returns>
			public override string ToString()
			{
				StringBuilder builder = new StringBuilder();
				builder.AppendLine(this.GetType().GetFullName() + ".Rotation: " + (this.Rotation.HasValue ? this.Rotation.Value.GetStringValue() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".Width: " + (this.Width.HasValue ? this.Width.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".Height: " + (this.Height.HasValue ? this.Height.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".Navigation: " + (this.Navigation.HasValue ? this.Navigation.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".ScreenOn: " + (this.ScreenOn.HasValue ? this.ScreenOn.Value.ToString() : "N/A"));
				builder.AppendLine(this.GetType().GetFullName() + ".AutoRotation: " + (this.AutoRotation.HasValue ? this.AutoRotation.Value.ToString() : "N/A"));

				return builder.ToString().TrimEnd();
			}
		}
	}
}
