﻿using AirFusion.Common;
using AirFusion.Manager.DeviceManager;
using System;
using System.Text;

namespace AirFusion.Manager.FeatureHelper
{
	/// <summary>
	/// Represents the event arguments of launch requested event.
	/// </summary>
	public class LaunchRequestedEventArgs : EventArgs
	{
		#region Properties

		/// <summary>
		/// Gets the device that requested.
		/// </summary>
		public AFUserDevice Device { get; private set; }

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes a new instance of the <see cref="LaunchRequestedEventArgs"/> class.
		/// </summary>
		internal LaunchRequestedEventArgs()
		{
			this.Device = null;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="LaunchRequestedEventArgs" /> class.
		/// </summary>
		/// <param name="device">The device of the requester.</param>
		internal LaunchRequestedEventArgs(AFUserDevice device)
		{
			this.Device = device;
		}

		/// <summary>
		/// Returns a <see cref="System.String" /> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String" /> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.AppendLine(this.GetType().GetFullName() + ".Device" + (this.Device != null ? ".InstanceID: " + this.Device.InstanceID.GetStringFormat() : ": N/A"));

			return builder.ToString().TrimEnd();
		}

		#endregion Methods
	}
}