#ifndef __RFBKEYSYM_H__
#define __RFBKEYSYM_H__

#include "stdhdrs.h"
#include "Log.h"
#include "KeyMap.h"
#include "ClientConnection.h"

class ClientConnection;

class RfbKeySym
{
public:
	RfbKeySym(ClientConnection *mclient, Log *mlog);
	virtual ~RfbKeySym();

	void processKeyEvent(unsigned short virtKey, unsigned int addKeyData);
	void processCharEvent(WCHAR charCode, unsigned int addKeyData);
	void processVirtualKey(unsigned short virtKey);	// [New Function]

private:
	void clearKeyState();

	void releaseMeta();
	void restoreMeta();
	void releaseModifier(unsigned char modifier);
	void restoreModifier(unsigned char modifier);
	void releaseModifiers();
	void restoreModifiers();

	bool isPressed(unsigned char virtKey);

	// This function does distinguish between a right or left modifier and
	// if the virtKey is a modifier (e.g VK_CONTROL), the function always return
	// a left- or right-hand virtual key value (e.g. VK_LCONTROL or VK_RCONTROL)
	// by taking into account the isRightHint flag.
	// If virtKey is not a modifier, the function returns virtKey value without changing.
	unsigned char distinguishLeftRightModifier(unsigned char virtKey, bool isRightHint);

	// Send one key event (Alt translated to Meta if Scroll Lock is on).
	virtual void sendKeySymEvent(unsigned int rfbKeySym, bool down);

	// Send one key event (Alt not translated to Meta).
	virtual void sendVerbatimKeySymEvent(unsigned int rfbKeySym, bool down);

	// [***][TightVNC v2.7.10] RemoteViewerCore.cpp: void RemoteViewerCore::sendKeyboardEvent(...)
	void sendKeyboardEvent(bool downFlag, UINT32 key);

	// This state doesn't difference between left and right modifiers. It's
	// needed to ToUnicodeEx().
	unsigned char m_viewerKeyState[256];

	// This state does difference between left and right modifiers. It's
	// needed to know the server side state (e.g. to release or restore
	// modifiyers state outside from a real key event).
	unsigned char m_serverKeyState[256];
	bool m_leftMetaIsPressed;
	bool m_rightMetaIsPressed;

	Keymap m_keyMap;
	bool m_allowProcessCharEvent;

	ClientConnection *m_client;
	Log *m_log;
};

#endif // __RFBKEYSYM_H__