// VNCHandler.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "VNCHandler.h"
#include <vector>
#include <thread>

#define MAX_SIZE_VNC_HANDLERS 5

// VNCHandler List
ClientConnection* vnc_list[MAX_SIZE_VNC_HANDLERS] = {0};
ClientConnection* delete_vnc_list[MAX_SIZE_VNC_HANDLERS] = {0};

VNCMessageCallback VncMessageCallbackFun = nullptr;
FileUploadProCallback FTUploadCallbackFun = nullptr;
FileDownloadProCallback FTDownloadCallbackFun = nullptr;

//void VNCTransferFileProcess(int vnc_index)
void VNCUploadFileProcess(int vnc_index, int file_index, int progress, int state)
{
	if(FTUploadCallbackFun != nullptr)
		FTUploadCallbackFun(vnc_index, file_index, progress, state);
}

void VNCDownloadFileProcess(int vnc_index, wchar_t* file_name, int progress, int state)
{
	if(FTDownloadCallbackFun != nullptr)
		FTDownloadCallbackFun(vnc_index, file_name, progress, state);
}

void ClientConnectionMessage(int vnc_id, int vnc_msg_type, int vnc_fun_code, int vnc_err_code, char* msg)
{
	if(VncMessageCallbackFun != nullptr)
		VncMessageCallbackFun(vnc_id, vnc_msg_type, vnc_fun_code, vnc_err_code, msg);

	//if(VncMessageCallbackFun != nullptr)
	//	std::thread thr(VncMessageCallbackFun, vnc_id, vnc_msg_type, vnc_fun_code, vnc_err_code, msg);
}

VNCAPI void WINAPI SetVNCCallbackFun(VNCMessageCallback callbackPointer)
{
	if(VncMessageCallbackFun == nullptr)
		VncMessageCallbackFun = callbackPointer;
}

VNCAPI void WINAPI SetFileTransferCallbackFun(FileUploadProCallback uploadCallback, FileDownloadProCallback downloadCallback)
{
	if(FTUploadCallbackFun == nullptr)
		FTUploadCallbackFun = uploadCallback;

	if(FTDownloadCallbackFun == nullptr)
		FTDownloadCallbackFun = downloadCallback;
}

VNCAPI void WINAPI EnableDebugLog(char* logfile_path, int iLogLevel)
{
	vnclog.SetLevel(iLogLevel);	//vnclog.SetLevel(6);
	vnclog.SetFile(logfile_path);
	vnclog.SetMode(Log::ToFile);
}

VNCAPI void WINAPI WriteDebugLog(int iLogLevel, char* sDebugMsg)
{
	vnclog.Print(iLogLevel, sDebugMsg);
}

VNCAPI void WINAPI CloseDebugLog()
{
	vnclog.SetMode(0);
}

VNCAPI void WINAPI SetDownloadFolderPath(BYTE bIndex, wchar_t* folder_path)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		vnc_list[iIndex]->SetDownloadFolderPath(folder_path);
	}
}

VNCAPI int WINAPI CreateNewVNCHandler()
{
	// Find the index of null pointer
	int iRet = -1;
	for(int i = 0; i < MAX_SIZE_VNC_HANDLERS; i++)
	{
		if(vnc_list[i] == NULL)
		{
			iRet = i;
			vnc_list[i] = new ClientConnection(i);
			vnc_list[i]->MessageCallback = ClientConnectionMessage;
			vnc_list[i]->SetFileTransferUpdateCallback(VNCUploadFileProcess, VNCDownloadFileProcess);

			break;
		}
	}

	// Clear the delete VNC list
	for(int i = 0; i < MAX_SIZE_VNC_HANDLERS; i++)
	{
		if(delete_vnc_list[i] != NULL)
		{
			delete delete_vnc_list[i];
			delete_vnc_list[i] = NULL;
		}
	}

	return iRet;	// If not find -> Return -1
}

VNCAPI DWORD WINAPI RunVNCHandler(BYTE bIndex, VNCConnectInfo mConnInfo)
{
	VNCErrorCode bRet = EC_NoError;

	int iIndex = (int) bIndex;
	vnclog.Print(5, _T("[VNCHandler] RunVNCHandler(): Start => index = %d\n"), iIndex);
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		// Set connection info
		vnc_list[iIndex]->SetConnectInfo(mConnInfo.sIPv4, (int)mConnInfo.uiPortNum, mConnInfo.sPassword);

		try
		{
			vnc_list[iIndex]->Run();
		}
		catch(VncException &e)
		{
			if(e.m_ErrType == EXP_CustomError)
				bRet = (VNCErrorCode)e.m_iErrCode;
			else if(e.m_ErrType == EXP_SocketError)
				bRet = EC_SocketError;
			else
				bRet = EC_OtherError;

			e.Report();
		}
		catch(WarningException &e)
		{
			e.Report();
		}
		catch (AuthException &e)
		{
			e.Report();
		}
		catch (Exception &e)
		{
			e.Report();
		}
	}

	vnclog.Print(5, _T("[VNCHandler] RunVNCHandler(): END => VNC err code = %d\n"), bRet);
	return (DWORD)bRet;
}

VNCAPI bool WINAPI InitVNCClipboard(BYTE bIndex, HWND hWnd)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		return vnc_list[iIndex]->InitClipboard(hWnd);
	}

	return false;
}

VNCAPI void WINAPI RemoveVNCClipboard(HWND hWnd)
{
	ClientConnection::RemoveClipboard(hWnd);
}

VNCAPI void WINAPI SetAFMiracastScreenInfo(BYTE bIndex, int iWidth, int iHeight, int iDeltaX, int iDeltaY, float fDpiRate)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		if(vnc_list[iIndex] != NULL)
			vnc_list[iIndex]->SetCoordinateTransformationInfo(iWidth, iHeight, iDeltaX, iDeltaY, fDpiRate);
	}
}

VNCAPI void WINAPI VncWinProc(BYTE bIndex, HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		try
		{
			if(vnc_list[iIndex] != NULL)
				vnc_list[iIndex]->WndProc(hwnd, iMsg, wParam, lParam);
		}
		catch(VncException &e)
		{
			e.Report();
		}
		catch(WarningException &e)
		{
			e.Report();
		}
		catch (ErrorException &e)
		{
			e.Report();
		}
		catch (Exception &e)
		{
			e.Report();
		}
	}
}

VNCAPI void WINAPI SendIMEChars(BYTE bIndex, wchar_t* wsBuffer, int iWsLeng)
{
	int iIndex = (int) bIndex;
	try
	{
		int len = WideCharToMultiByte (CP_UTF8, 0, (LPCWSTR)wsBuffer , -1, NULL, 0, NULL, NULL);
		char* unixContents = new char[len + 1];
		ZeroMemory (unixContents, len + 1);
		WideCharToMultiByte (CP_UTF8, 0, (LPCWSTR)wsBuffer, -1, unixContents, len, NULL, NULL);

		if(vnc_list[iIndex] != NULL)
			vnc_list[iIndex]->SendClientInputText(unixContents, len + 1);

		delete[] unixContents;
	}
	catch(VncException &e)
	{
		if(e.m_ErrType == EXP_SocketError)
			ClientConnectionMessage(iIndex, MT_ErrorMessage, FC_SendIMEString, EC_SocketError, nullptr);
		else
			ClientConnectionMessage(iIndex, MT_ErrorMessage, FC_SendIMEString, EC_OtherError, nullptr);
	}
}

VNCAPI void WINAPI SendVirtualKey(BYTE bIndex, unsigned int uiVirtualKeyCode, unsigned int uiKeyStateFlg)
{
	// uiKeyStateFlag:
	// - KEYEVENTF_KEYDOWN = 0x0000,
	// - KEYEVENTF_EXTENDEDKEY = 0x0001,
	// - KEYEVENTF_KEYUP = 0x0002,

	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		if(vnc_list[iIndex] != NULL)
		{
			if(uiKeyStateFlg == 0x0)
			{
				if(vnc_list[iIndex] != NULL)
					vnc_list[iIndex]->sendVirtualKey(uiVirtualKeyCode);
			}
		}
	}
}

VNCAPI void WINAPI SendTouchControl(BYTE bIndex, unsigned int uiTouchEventType, long point_x, long point_y)
{
	// C#: VNCClient.cs defines
	//	public enum TouchEventType
	//	{
	//		Download = 0,
	//		Move = 1,
	//		Up = 2,
	//		SwipeLeftToRight = 3,
	//		SwipeRightToLeft = 4,
	//		DropDownNotification = 5
	//	}
	// [*]Note: The value of point_x & point_y has been based on the current DPI to transfer.

	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		if(vnc_list[iIndex] != NULL)
		{
			switch (uiTouchEventType)
			{
			case 0:
				vnc_list[iIndex]->sendTouchDown(point_x, point_y);
				break;
			case 1:
				vnc_list[iIndex]->sendTouchMove(point_x, point_y);
				break;
			case 2:
				vnc_list[iIndex]->sendTouchUp(point_x, point_y);
				break;
			case 3:
				vnc_list[iIndex]->SwipeLeftToRight();
				break;
			case 4:
				vnc_list[iIndex]->SwipeRightToLeft();
				break;
			case 5:
				vnc_list[iIndex]->DropDownNotification();
				break;
			default:
				break;
			}
		}
	}
}

VNCAPI void WINAPI TransferFileToServer(BYTE bIndex, wchar_t* wsClientFilePath, int iFileId)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		if(vnc_list[iIndex] != NULL)
		{
			vnc_list[iIndex]->SendFileToServer(wsClientFilePath, iFileId);
		}
	}
}

VNCAPI void WINAPI CloseVNCHandler(int iVncIndex)
{
	if((iVncIndex < MAX_SIZE_VNC_HANDLERS) && (iVncIndex >= 0))
	{
		if(vnc_list[iVncIndex] == NULL)
			return;

		//vnc_list[iVncIndex]->MessageCallback = nullptr;
		vnc_list[iVncIndex]->Close();

		//delete vnc_list[iVncIndex];
		for(int i = 0; i< MAX_SIZE_VNC_HANDLERS; i++)
		{
			if(delete_vnc_list[i] == NULL)
			{
				delete_vnc_list[i] = vnc_list[iVncIndex];
				break;
			}
		}

		vnc_list[iVncIndex] = NULL;
	}
}

VNCAPI void WINAPI ReleaseVNCHandlers()
{
	for(int i = 0; i < MAX_SIZE_VNC_HANDLERS; i++)
	{
		CloseVNCHandler(i);
	}
}

VNCAPI void WINAPI RightClickClipboard(BYTE bIndex)
{
	int iIndex = (int) bIndex;
	if(iIndex < MAX_SIZE_VNC_HANDLERS)
	{
		if(vnc_list[iIndex] != NULL)
		{
			if(vnc_list[iIndex] != NULL)
				vnc_list[iIndex]->pasteClipboardText();
		}
	}
}